import pygame

class GFXManager:
	def __init__(self, title, screenSize = (800, 600), fullScreen = False):
		pygame.init()
		f = pygame.FULLSCREEN if fullScreen else 0
		self.screen = pygame.display.set_mode(screenSize, pygame.DOUBLEBUF|f)
		pygame.display.set_caption(title)
		self.image = {}
		self.frames = 0
		self.ticks = pygame.time.get_ticks()
		self.font = None
		self.fontSignature = None
	
	def beginFrame(self):
		self.screen.fill((0,0,0))
	
	def endFrame(self):
		pygame.display.flip()
		self.frames += 1
	
	def getFps(self):
		return (self.frames * 1000) / (pygame.time.get_ticks() - self.ticks)
	
	def draw(self, image, x, y):
		self.screen.blit(self.getImage(image), (x, y))
	
	def getImage(self, name):
		if name not in self.image:
			i = self.loadImage(name)
			self.image[name] = i
		return self.image[name]
	
	def loadImage(self, file):
		try:
			image = pygame.image.load(file)
		except pygame.error, message:
			print "Failed to load image: %s" % file
			raise SystemExit, message
		return image.convert_alpha()
	
	def line(self, color, startPos, endPos, width = 1):
		pygame.draw.line(self.screen, color, startPos, endPos, width)
	
	def rect(self, color, rect):
		self.screen.fill(color, rect)
	
	def ellipse(self, color, rect):
		pygame.draw.ellipse(self.screen, color, rect)
	
	def loadFont(self, filename, size):
		self.font = pygame.font.Font(filename, size)
		self.fontSignature = filename + str(size)
	
	def write(self, text, color, x, y):
		if self.font != None:
			textName = str(text) + str(color) + self.fontSignature
			if textName not in self.image:
				t = self.font.render(str(text), True, color)
				self.image[textName] = t
			self.draw(textName, x, y)
	
	def textWidth(self, text):
		if self.font != None:
			color = (255, 255, 255)
			textName = str(text) + str(color) + self.fontSignature
			if textName not in self.image:
				t = self.font.render(str(text), True, color)
				self.image[textName] = t
			else:
				t = self.getImage(textName)
			return t.get_width()
