import pygame
import random

from gfxman import *

class FlyingWord:
	wordList = None
	
	def __init__(self, gfx, x, y):
		self.word = FlyingWord.randomWord()
		self.width = gfx.textWidth(self.word)
		self.x = float(x)
		self.y = y
		self.pos = 0
		self.active = False
		self.failTimer = 0
	
	def nextChar(self):
		return self.word[self.pos]
	
	def isComplete(self):
		if self.pos == len(self.word):
			return True
		return False
	
	@classmethod
	def randomWord(cls):
		if FlyingWord.wordList == None:
			FlyingWord.loadWords()
		return random.choice(FlyingWord.wordList)
	
	@classmethod
	def loadWords(cls):
		FlyingWord.wordList = []
		with open("words.txt") as f:
			for line in f:
				line = line.strip().lower()
				if line.isalpha() and len(line) > 2 and len(line) < 20:
					FlyingWord.wordList.append(line)
				else:
					print "Word '%s' rejected." % line
		if len(FlyingWord.wordList) < 5:
			raise SystemExit, "Not enough words."
	
	def update(self, delta, combo):
		self.x -= (delta / 50.0) * (1.0 + (combo / 10.0))
		if self.x + self.width > 800:
			self.x -= delta / 50.0
		if self.failTimer > 0:
			self.failTimer -= delta
			if self.failTimer < 0:
				self.failTimer = 0
	
	def isDead(self, wallPos):
		if self.x - 10.0 < wallPos:
			return True
		return False
	
	def draw(self, gfx):
		if self.active == False:
			if self.failTimer == 0:
				gfx.ellipse((64, 0, 128), (int(self.x) - 10, self.y - 10, self.width + 20, 18 + 20))
				gfx.write(self.word, (128, 128, 255), int(self.x), self.y)
			else:
				gfx.ellipse((128, 0, 0), (int(self.x) - 10, self.y - 10, self.width + 20, 18 + 20))
				gfx.write(self.word, (255, 128, 128), int(self.x), self.y)
		else:
			gfx.ellipse((0, 128, 0), (int(self.x) - 10, self.y - 10, self.width + 20, 18 + 20))
			done = self.word[:self.pos]
			notDone = self.word[self.pos:]
			gfx.write(done, (128, 255, 128), int(self.x), self.y)
			gfx.write(notDone, (128, 128, 255), int(self.x) + gfx.textWidth(done), self.y)

def main():
	gfx = GFXManager("Typo of Doom")
	
	gfx.loadFont("freemono.ttf", 18)
	
	wallPos = 0.0
	
	words = []
	wordTimer = 0
	activeWord = None
	inputBuffer = []
	comboCounter = 0
	combo = 1
	score = 0
	showScore = 0
	scoreAdder = 1
	nextY = 18 * 2
	
	comboTable = []
	for c in xrange(9):
		comboTable.append((10 - c) ** 1.6)
	
	wallSpeedTable = []
	for i in xrange(25):
		wallSpeedTable.append(0.01 * ((i + 2) ** 1.25))
	
	scoreTable = []
	for i in xrange(20):
		scoreTable.append(i ** 1.5)
	
	startTime = pygame.time.get_ticks()
	oldTime = startTime
	end = False
	while not end:
		for event in pygame.event.get():
			if event.type == pygame.QUIT or (event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE):
				end = True
			if event.type == pygame.KEYUP:
				if event.key >= pygame.K_a and event.key <= pygame.K_z:
					inputBuffer.append(chr(event.key))
		
		now = pygame.time.get_ticks()
		delta = now - oldTime
		oldTime = now
		
		wordTimer -= delta
		if (wordTimer <= 0 and len(words) < 8) or len(words) == 0:
			curWords = []
			for w in words:
				curWords.append(w.word)
			w = FlyingWord(gfx, 810, nextY)
			while w.word in curWords:
				w = FlyingWord(gfx, 810, nextY)
			words.append(w)
			nextY += 50
			if nextY > (600 - 28):
				nextY -= (600 - 28 - 18 * 2)
			wordTimer += 1500
			if wordTimer > 1500:
				wordTimer = 1500
		
		combo = 1
		for c in xrange(9):
			if comboCounter > comboTable[c]:
				combo = (10 - c)
				break
		
		if len(inputBuffer) > 0:
			key = inputBuffer.pop()
			if activeWord not in words:
				activeWord = None
			if activeWord != None:
				if key == activeWord.nextChar():
					activeWord.pos += 1
					if activeWord.isComplete():
						n = scoreTable[len(activeWord.word)]  * combo
						score += int(n * 10.0)
						wallPos -= 100.0 + 3.0 * n
						if wallPos < 0:
							wallPos = 0.0
						words.remove(activeWord)
						activeWord = None
						comboCounter += 1
				else:
					activeWord.active = False
					activeWord.pos = 0
					activeWord.failTimer = 1000
					activeWord = None
					comboCounter = 0
			else:
				found = False
				for w in words:
					if w.failTimer == 0:
						if key == w.nextChar():
							w.active = True
							activeWord = w
							activeWord.pos += 1
							found = True
							break
				if found == False:
					comboCounter = 0
		
		wallPos += float(delta) * wallSpeedTable[min(24, int((now - startTime) / 15000))]
		if (wallPos / 2.0) >= 800:
			end = True
		
		for w in words:
			w.update(delta, combo)
			if w.isDead(wallPos / 2.0):
				if activeWord == w:
					activeWord = None
				words.remove(w)
				comboCounter = 0
		
		if score > showScore:
			showScore += scoreAdder
			if showScore > score:
				showScore = score
			scoreAdder += 1
		else:
			scoreAdder = 1
		
		gfx.beginFrame()
		
		for y in xrange(25, 600, 25):
			gfx.line((192, 0, 0), (0, y), (int(wallPos / 2.0), y))
		gfx.line((255, 0, 0), (int(wallPos / 2.0), 0), (int(wallPos / 2.0), 600), 5)
		
		for w in words:
			w.draw(gfx)
		
		gfx.write(showScore, (192, 192, 255), 800 - gfx.textWidth(score), 0)
		if combo >= 2:
			text = "x%d" % combo
			gfx.write(text, (192, 255, 192), 800 - gfx.textWidth(text), 18)
		
		gfx.endFrame()
	
	print score

if __name__ == "__main__":
	main()